/*

Thumbnailator - Thumbnail generation library

Copyright (c) 2008, Chris Kroells
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of its contributors may be
      used to endorse or promote products derived from this software without
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


package net.coobird.thumbnailator;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.imageio.*;
import javax.imageio.stream.*;


/**
 * The <code>TGenerator</code> class is the thumbnail generating class
 * of Thumbnailator.
 * Thumbnails are generated according to the parameters passed into the
 * <code>TGenerator</code> by a {@link TImageParams} object.
 * The {@link THelper} and {@link TRunner} classes can be used to facilitate
 * the use of <code>TGenerator</code>.
 */
public final class TGenerator
{
	// Constants used for specifying the rotation of the thumbnail.
	/** No rotatation applied to thumbnail. */
	public static final int NO_ROTATION = 0;
	/** Clockwise 90 degree rotation applied to thumbnail. */
	public static final int ROTATE_90_CW = 1;
	/** Counter-clockwise 90 degree rotation applied to thumbnail. */
	public static final int ROTATE_90_CCW = 2;
	/** 180 degree rotation applied to thumbnail. */
	public static final int ROTATE_180 = 3;
		
	// Watermark positioning on the thumbnail.
	/** Positions the watermark at top-left corner. */
	public static final int WM_TOP_LEFT = 0;
	/** Positions the watermark at top-right corner. */
	public static final int WM_TOP_RIGHT = 1;
	/** Positions the watermark at bottom-left corner. */
	public static final int WM_BOTTOM_LEFT = 2;
	/** Positions the watermark at bottom-left corner. */
	public static final int WM_BOTTOM_RIGHT = 3;
	/** Positions the watermark in the center. */
	public static final int WM_CENTER = 4;
	/** Positions the watermark at an arbituary position. */
	public static final int WM_ARBITUARY = 5;
	/** Undefined positioning for watermark. */
	public static final int WM_UNDEFINED = -1;

	// Rendering quality mode of the thumbnail.
	/** Renders the thumbnail at the highest quality. */
	public static int RENDER_HIGHEST_QUALITY = 0;
	/** Renders the thumbnail at the high quality. */
	public static int RENDER_HIGH_QUALITY = 1;
	/** Renders the thumbnail at the medium quality. */
	public static int RENDER_MEDIUM_QUALITY = 2;
	/** Renders the thumbnail at the low quality. */
	public static int RENDER_LOW_QUALITY = 3;
	/**
	 * Rendering of the thumbnail will be optimized for speed.
	 * This mode offers the highest speed rendering at the expense of
	 * the thumbnail quality.
	 */
	public static int RENDER_FAST = 4;
	/**
	 * Renders the thumbnail at a divisible multiple of the original size
	 * of the source image.
	 * This option reduces the time required for rendering.
	 */
	public static int RENDER_MULTIPLE = 5;
	
	
	// Specifies which image to load in readImage method
	private static int IMAGE_ORIGINAL = 0;
	private static int IMAGE_WATERMARK = 1;
	
	// Hashtable to hold ImageReaders/-Writers for each format.
	private Hashtable imageReadersTable;
	private Hashtable imageWritersTable;

	
	/**
	 * Instantiates a <code>TGenerator</code> object.
	 */
	public TGenerator()
	{
		String[] names = null;
		
		// Get the supported read file formats, and populate the
		// ImageReaders table with the first Reader.
		names = ImageIO.getReaderFormatNames();
		imageReadersTable = new Hashtable();
		
		for (int i = 0; i < names.length; i++)
		{
			Iterator iter = ImageIO.getImageReadersByFormatName(names[i]);
			if (iter.hasNext())
				imageReadersTable.put((Object)names[i], iter.next());	
		}
		
		// Get the supported writer file formats, and populate the
		// ImageWriters table with the first Writer.
		names = ImageIO.getWriterFormatNames();
		imageWritersTable = new Hashtable();

		for (int i = 0; i < names.length; i++)
		{
			Iterator iter = ImageIO.getImageWritersByFormatName(names[i]);
			if (iter.hasNext())
				imageWritersTable.put((Object)names[i], iter.next());	
		}
	}

	
	/**
	 * Generates a thumbnail from parameters specified in a
	 * {@link TImageParams} object.
	 * @param param 				the parameters for creating the thumbnail
	 * 								as a <code>TImageParams</code> object.
	 * @throws IOException 			Thrown when a problem occurs during the
	 * 								thumbnail generation processing.
	 * 								This exception is thrown from the
	 * 								{@link javax.imageio.ImageIO} classes.
	 */
	public void createThumbnail(TImageParams param) throws IOException
	{
		// Store for the thumbnail image.
		BufferedImage thumbnailImage;
		
		// Fill in the inputFormat information
		if (param.getInputImageFile() != null && param.getInputFormat() == null)
			param.setInputFormat(TImageParams.determineFormat(param.getInputImageFile()));
		// Fill in the outputFormat information
		if (param.getOutputImageFile() != null && param.getOutputFormat() == null)
			param.setOutputFormat(TImageParams.determineFormat(param.getOutputImageFile()));

		// Generate thumbnail, retrieve as BufferedImage.
		thumbnailImage = generateThumbnailImage(param);
		
		// If outputImageFile is specified, then write image to disk
		// If outputImage is specified, then change the param's outputImage
		if (param.getOutputImageFile() != null)
		{
			// Output the thumbnail to outFile.
			writeImage(thumbnailImage, param);
			
			// Sets the last modified time of the thumbnail to the lastmod
			// time of the original image file.
			if (param.isKeepDateTime() && param.getInputImageFile() != null)
			{
				long lastmod = param.getInputImageFile().lastModified();
				param.getOutputImageFile().setLastModified(lastmod);
			}
		}
		else
		{
			// Sets the thumbnail.
			param.setOutputImage(thumbnailImage);
		}
	}
		
	/**
	 * Generates a thumbnail as an {@link BufferedImage} from the parameters
	 * specified from the {@link TImageParams} object. 
	 * @param param 				the parameters for thumbnail generation as
	 * 								a <code>TImageParams</code> object
	 * @return 						the thumbnail as an instance of
	 * 								<code>BufferedImage</code>.
	 * @throws IOException 			Thrown when a problem occurs during the
	 * 								thumbnail generation processing.
	 * 								This exception is thrown from the
	 * 								{@link javax.imageio.ImageIO} classes.
	 */
	private BufferedImage generateThumbnailImage(TImageParams param) throws IOException
	{
		// Sets the input image
		BufferedImage img = null;
		
		// Sets the img BufferedImage from the source
		if (param.getInputImage() != null)
		{
			img = param.getInputImage();
		}
		else if (param.getInputImageFile() != null)
		{
			img = readImage(IMAGE_ORIGINAL, param);
		}
		else
		{
			throw new IllegalArgumentException("No input file or image.");
		}
		
		if (param.getOutputRenderQuality() == RENDER_MULTIPLE)
			return img; 
		
		BufferedImage timg = null;
		BufferedImage wmimg = null;
		
		// Local variables used in method
		File inputWatermarkFile = param.getInputWatermarkFile();
		Dimension outputDimension = param.getOutputDimension();
		int outputOrientation = param.getOutputOrientation();
		int watermarkPosition = param.getWatermarkPosition();
		Rectangle watermarkRectangle = param.getWatermarkRectangle();
		boolean keepAspectRatio = param.isKeepAspectRatio();
		int outputRenderQuality = param.getOutputRenderQuality();
		float watermarkAlpha = param.getWatermarkAlpha();

		// Adjust the output dimension if the aspect ratio needs
		// to be preserved.
		if (keepAspectRatio)
			outputDimension = getScaledDimension(img.getWidth(),
													img.getHeight(),
													outputDimension);
		
		// If the inputWatermarkFile is specified, then
		// read the watermark, do calculations for position and size.
		if (inputWatermarkFile != null)
		{
			// Read the watermark
			wmimg = readImage(IMAGE_WATERMARK, param);
	
			// Determines the dimension of the watermark.
			// Checks if a resize is necessary, depending on whether or not
			// to maintain the aspect ratio.
			Dimension dim = null;
			
			if (keepAspectRatio)
				dim = getScaledDimension(wmimg.getWidth(), wmimg.getHeight(),
											param.getWatermarkDimension());
			else
				dim = new Dimension(watermarkRectangle.width,
									watermarkRectangle.height);
			
			// If there is no specification of size of watermark, then
			// set it to the original size of the watermark.
			if (watermarkRectangle.width == 0 && watermarkRectangle.height == 0)
				dim = new Dimension(wmimg.getWidth(), wmimg.getHeight());
			
			// Calculates the position of the watermark.
			Point pos = calcWatermarkPosition(outputDimension.width,
												outputDimension.height,
												dim.width,
												dim.height,
												outputOrientation,
												watermarkPosition);

			// However, if WM_ARBITUARY, reset the position to specified.
			if (watermarkPosition == WM_ARBITUARY)
				pos = new Point(watermarkRectangle.x, watermarkRectangle.y);
			
			// Determines the position and size of watermark.
			watermarkRectangle = new Rectangle(pos.x, pos.y, 
												dim.width, dim.height);
		}
		else if (param.getInputWatermark() != null)
		{
			wmimg = param.getInputWatermark();
		}
		else
		{
			// No watermark present.
			wmimg = null;
		}
		
		// Resize watermark, if necessary.
		if (wmimg != null)
		{
			// HQ watermark resize, if warranted.
			if (wmimg.getWidth() == watermarkRectangle.width ||
					wmimg.getHeight() == watermarkRectangle.height)
			{
				// No resize needed.
				// wmimg has been read already previously.
				// This block can be blank, intentionally.					
			}
			else // inputWatermarkFile == null
			{
				// resize needed, so do it in HQ mode
				wmimg = resizeImage(wmimg,
									watermarkRectangle.width,
									watermarkRectangle.height,
									NO_ROTATION, outputRenderQuality);
			}
		}
		
		// Check if we may go out of memory, if so, then MEDIUM quality.
		// Assume 4-byte per pixel.
		int size = (outputDimension.width * 2) *
					(outputDimension.height * 2) * 4;
				
		if (size > Runtime.getRuntime().freeMemory())
			if (outputRenderQuality == RENDER_HIGHEST_QUALITY ||
					outputRenderQuality == RENDER_HIGH_QUALITY)
				outputRenderQuality = RENDER_MEDIUM_QUALITY;

		
		// Choose thumbnail generation based on rendering quality
		if (outputRenderQuality == RENDER_HIGHEST_QUALITY ||
				outputRenderQuality == RENDER_HIGH_QUALITY)
		{
			// Make temporary image (tximg)
			// This is used to smooth out image by first creating a
			// temporary image that is double the destination image,
			// using high-quality rendering.

			// Double Size the original image
			// Enable for HIGHEST and HIGH
			BufferedImage tximg = resizeImage(img,
												outputDimension.width * 2,
												outputDimension.height * 2,
												NO_ROTATION, outputRenderQuality);
			
			// Clean up
			img.flush();
			img = null;
			
			// Resize the double-sized image to destination size.
			timg = resizeImage(tximg,
								outputDimension.width, outputDimension.height,
								outputOrientation, outputRenderQuality);
			
			// Clean up
			tximg.flush();
			tximg = null;
		}
		else if (outputRenderQuality == RENDER_MULTIPLE)
		{
			// Rather than resizing, use the original image subsampled to
			// a multiple.
			timg = img;

			// Clean up.
			img.flush();
			img = null;

			// Load watermark.
			if (inputWatermarkFile != null)
				wmimg = readImage(IMAGE_WATERMARK, param);
		}
		else // if (qual == RENDER_MEDIUM_QUALITY || qual == RENDER_LOW_QUALITY)
		{
			// Render in low-quality:
			// * Skips the enlargement of image step
			// * Uses low-quality resizeImage() method
			
			// make thumbnail, low-quality mode
			timg = resizeImage(img,
					outputDimension.width, outputDimension.height,
					outputOrientation, outputRenderQuality);
			
			// clean up
			img.flush();
			img = null;
			
			// load watermark
			if (inputWatermarkFile != null)
				wmimg = readImage(IMAGE_WATERMARK, param);
		}
		
		
		// Draw the watermark.
		if (wmimg != null)
		{	
			// add watermark
			Graphics2D g = timg.createGraphics();
			AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, watermarkAlpha);
			g.setComposite(ac);
			g.drawImage(wmimg, watermarkRectangle.x, watermarkRectangle.y,
						watermarkRectangle.width, watermarkRectangle.height, null);
			g.dispose();
		}
		
		// clean up
		if (wmimg != null)
		{
			wmimg.flush();
			wmimg = null;
		}
		
		// Returns the rendered thumbnail image.
		return timg;
	}

	/**
	 * Reads an image from a file, specified by the parameters held in a
	 * {@link TImageParams} object, and returns it as a {@link BufferedImage}
	 * object.
	 * @param image 				specifies the type of image being read,
	 * 								either the original image
	 * 								({@link TGenerator#IMAGE_ORIGINAL}) or the 
	 * 								watermark image
	 * 								({@link TGenerator@IMAGE_WATERMARK}).
	 * @param param 				the <code>TImageParam</code> object holding
	 * 								the parameters of the thumbnail to be
	 * 								produced
	 * @return 						the image read from file as a
	 * 								<code>BufferedImage</code> object.
	 * @throws IOException 			Thrown when a problem occurs when
	 * 								attempting to read an image.
	 * @throws IllegalArgumentException 	Thrown when the input file format
	 * 										of the <code>TImageParams</code>
	 * 										contains an unsupported image
	 * 										file format.
	 */
	private BufferedImage readImage(int image, TImageParams param)
									throws IOException
	{
		String format = null;
		File sourceFile = null;
		boolean allowSubsampling = param.isAllowSubsampling();
		
		// Allow subsampling for RENDER_MULTIPLE
		if (param.getOutputRenderQuality() == RENDER_MULTIPLE)
			allowSubsampling = true;
		
		// Sets the file format and source file depending on reading in
		// the original image or watermark image
		if (image == IMAGE_ORIGINAL)
		{
			sourceFile = param.getInputImageFile();
			format = param.getInputFormat();
			
			if (format == null)
				format = TImageParams.determineFormat(sourceFile);
		}
		else if (image == IMAGE_WATERMARK)
		{
			sourceFile = param.getInputWatermarkFile();
			format = TImageParams.determineFormat(sourceFile);
		}
		else
		{
			throw new IllegalArgumentException("Invalid image load type.");
		}
		
		// Get the ImageReader for the specified file format.
		ImageReader ir = null;

		if (imageReadersTable.containsKey(format))
			ir = (ImageReader)imageReadersTable.get(format);
		else
			throw new IllegalArgumentException("Input format not supported: " + format);
		
		// Opens the FileImageInputStream and set it as the
		// ImageReader's InputStream.
		FileImageInputStream fiis = new FileImageInputStream(sourceFile);
		ir.setInput(fiis);
		
		// Get the image input parameters
		ImageReadParam irParam = ir.getDefaultReadParam();
		
		// Sets the subsampling
		if (allowSubsampling)
		{
			int newWidth = param.getOutputWidth();
			int newHeight = param.getOutputHeight();
			int curWidth = ir.getWidth(0);
			int curHeight = ir.getHeight(0);
			
			double cnWidthRatio = 0;
			double cnHeightRatio = 0;
			
			int qualityFactor = 1;
			int qual = param.getOutputRenderQuality();

			if (qual == RENDER_HIGHEST_QUALITY)
				qualityFactor = 8;
			else if (qual == RENDER_HIGH_QUALITY)
				qualityFactor = 4;
			else if (qual == RENDER_MEDIUM_QUALITY)
				qualityFactor = 3;
			else if (qual == RENDER_LOW_QUALITY)
				qualityFactor = 2;
			else if (qual == RENDER_FAST)
				qualityFactor = 1;
			else if (qual == RENDER_MULTIPLE)
				qualityFactor = 1;
			
			
			// Calculate the ratio between the new and original
			// dimensions
			if (param.getOutputOrientation() == TGenerator.NO_ROTATION ||
					param.getOutputOrientation() == TGenerator.ROTATE_180)
			{
				cnWidthRatio = curWidth / newWidth;
				cnHeightRatio = curHeight / newHeight;
			}
			else
			{
				cnWidthRatio = curWidth / newHeight;
				cnHeightRatio = curHeight / newWidth;
			}
			
			double smallerRatio = Math.min(cnWidthRatio, cnHeightRatio);
			int subsample = (int)(smallerRatio / qualityFactor);
						
			if (subsample < 1)
				subsample = 1;
			
			irParam.setSourceSubsampling(subsample, subsample, 0, 0);
		}
		
		// Sets the destination of the image.
		IIOImage iioImage = ir.readAll(0, irParam);
		
		// Closes the ImageInputStream to free the file.
		fiis.close();
				
		// Retrieves the loaded image
		BufferedImage bi = (BufferedImage)iioImage.getRenderedImage();
		
		return bi;
	}
	
	/**
	 * Writes a thumbnail image to a file.
	 * @param bi 					the thumbnail image to write
	 * @param param 				the {@link TImageParams} object holding the
	 * 								parameters for the thumbnail to generate
	 * @throws IOException 			Thrown when a problem occurs when
	 * 								attempting to write the image to a file.
	 * 								The exception is thrown by 
	 * 								{@link FileImageOutputStream}.
	 * @throws IllegalArgumentException		Thrown when the output file format
	 * 										of the <code>TImageParams</code>
	 * 										contains an unsupported image file
	 * 										format string, or if the
	 * 										compression quality is out
	 * 										of range.
	 */
	private void writeImage(BufferedImage bi, TImageParams param)
							throws IOException
	{
		float quality = param.getOutputQuality();
		String format = param.getOutputFormat();
		File outFile = param.getOutputImageFile();
		
		// If the quality specified is not [0,1], then throw an
		// IllegalArgumentException.
		if (quality < 0.0f || quality > 1.0f)
			throw new IllegalArgumentException("Output compression quality must be between 0.0f and 1.0f, or Float.NaN.");
				
		// Retrieve ImageWriters that support the specified format.
		ImageWriter iw = null;
		
		if (imageWritersTable.containsKey(format))
			iw = (ImageWriter)imageWritersTable.get(format);
		else
			throw new IllegalArgumentException("Output format not supported: " + format);

		// If the output file exists, delete it.
		if (outFile.exists())
			outFile.delete();
		
		// Sets the ImageWriter's ImageOutputStream.
		FileImageOutputStream fios = new FileImageOutputStream(outFile);
		iw.setOutput(fios);
		
		// Gets the default parameters for writing an image.
		ImageWriteParam iwParam = iw.getDefaultWriteParam();
		
		// Checks if compression is supported.
		// This check is in place to prevent a UnsupportedFormatException
		// from being thrown when attempting to setCompressionMode.
		if (iwParam.canWriteCompressed())
		{
			// Allow the configuration of compression parameters.
			iwParam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);

			// Retrieves the default compression quality.
			if (quality == Float.NaN)
				quality = iwParam.getCompressionQuality();
			
			// Sets the compression quality.
			iwParam.setCompressionQuality(quality);
		}
			
		// Writes the image file.
		iw.write(null, new IIOImage(bi, null, null), iwParam);
		
		// Closes the ImageOutputStream to free the file.
		fios.close();
	}
	
	/**
	 * Calculates the position of the watermark.
	 * @param imgWidth 				the width of the thumbnail
	 * @param imgHeight				the height of the thumbnail
	 * @param wmWidth 				the width of the watermark
	 * @param wmHeight				the height of the watermark
	 * @param rot 					the rotation setting for the thumbnail
	 * @param wmPos 				the position of the watermark.
	 * @return 						position of the watermark returned as a
	 * 								{@link java.awt.Point} object.
	 */
	private Point calcWatermarkPosition(int imgWidth, int imgHeight,
										int wmWidth, int wmHeight,
										int rot, int wmPos)
	{
		// Determine the position for the watermark.
		int x = 0;
		int y = 0;
		
		// If rotated, swap the image dimensions, as they will change.
		if (!(rot == NO_ROTATION || rot == ROTATE_180))
		{
			int tmp = imgHeight;
			imgHeight = imgWidth;
			imgWidth = tmp;
		}

		// Calculate the position of the watermark depending on where
		// the watermark is to be situated.
		if (wmPos == WM_TOP_LEFT)
		{
			x = 0;
			y = 0;
		}
		else if (wmPos == WM_TOP_RIGHT)
		{
			x = imgWidth - wmWidth;
			y = 0;
		}
		else if (wmPos == WM_BOTTOM_LEFT)
		{
			x = 0;
			y = imgHeight - wmHeight;
		}
		else if (wmPos == WM_BOTTOM_RIGHT)
		{
			x = imgWidth - wmWidth;
			y = imgHeight - wmHeight;
		}
		else if (wmPos == WM_CENTER)
		{
			x = (imgWidth / 2) - (wmWidth / 2);
			y = (imgHeight / 2) - (wmHeight / 2);
		}
		
		return new Point(x, y);
	}

	/**
	 * Creates a thumbnail from the given {@link BufferedImage} instance and
	 * the dimensions specified.
	 * @param img					<code>BufferedImage</code> containing the
	 * 								source image.
	 * @param width 				the desired width of the thumbnail
	 * @param height				the desired height of the thumbnail
	 * @param outputOrientation 	the rotation to apply to the thumbnail
	 * @param outputRenderQuality 	the quality setting for thumbnail rendering
	 * @return						A thumbnail as a new instance of
	 * 								<code>BufferedImage</code> with the
	 * 								specified dimensions.
	 */
	private BufferedImage resizeImage(BufferedImage img, int width,
										int height, int outputOrientation,
										int outputRenderQuality)
	{
		BufferedImage thumbnailImage = null;
		int type = img.getType();
		
		// If returned a type 0 (which will cause a IllegalArgumentException
		// when creating a BufferedImage), set the type to ARGB.
		if (type == 0)
			type = BufferedImage.TYPE_INT_ARGB;
		
		// Create the thumbnail image with same type as the original image
		// swap the width and height if 90 degree rotation is applied.
		if (outputOrientation == NO_ROTATION || outputOrientation == ROTATE_180)
			thumbnailImage = new BufferedImage(width, height, type);
		else
			thumbnailImage = new BufferedImage(height, width, type);
			
		// Create the Graphics2D object used to draw the thumbnail.
		Graphics2D tg2 = thumbnailImage.createGraphics();
		
		// Choose whether to enable high-quality setting or not.
		if (outputRenderQuality != RENDER_FAST)
		{
			// High quality rendering hints.
			tg2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			tg2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
			tg2.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
			tg2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		}
		else // if (RENDER_FAST)
		{
			// Low quality rendering hints.
			tg2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
			tg2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
			tg2.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_SPEED);
			tg2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
		}
		
		// Rotate image
		if (outputOrientation == ROTATE_90_CW)
			tg2.rotate(Math.toRadians(90), (double)(height / 2), (double)(height / 2));
		else if (outputOrientation == ROTATE_90_CCW)
			tg2.rotate(Math.toRadians(270), (double)(width / 2), (double)(width / 2));
		else if (outputOrientation == ROTATE_180)
			tg2.rotate(Math.toRadians(180), (double)(width / 2), (double)(height / 2));
	
		// Make the thumbnail
		tg2.drawImage(img, 0, 0, width, height, null);
		img = null;
		
		// Clean up
		tg2.dispose();
		
		// Return newly created thumbnail
		return thumbnailImage;
	}

	/**
	 * Calculates the new thumbnail dimensions, preserving aspect ratio of the
	 * original image for the thumbnail, given the original dimensions and the
	 * target dimensions. The thumbnail generated will be contrained by the
	 * desired target dimension. For example, if the orignal image is 800 x 600
	 * and the target dimension is set to 1000 x 300, the resulting thumbnail
	 * dimension will be set to 400 x 300.
	 * @param imgWidth 				the width of the original image
	 * @param imgHeight 			the height of the original image
	 * @param targetDim 			the target dimension of the thumbnail, used
	 * 								as constraint for the thumbnail image size
	 * @param d 					the {@link Dimension} object containing the
	 * 								desired dimension of thumbnail.
	 * @return						Returns the thumbnail dimensions,
	 * 								preserving the aspect ratio from the
	 * 								original image.
	 */	
	private Dimension getScaledDimension(int imgWidth, int imgHeight,
											Dimension targetDim)
	{
		// Get the target Dimensions.
		int width = targetDim.width;
		int height = targetDim.height;
		
		// Calculate the scaling factors.
		double wscale = (double)width / (double)imgWidth;
		double hscale = (double)height / (double)imgHeight;
		
		// Let's see which side is larger..
		boolean isWidthLarger = (wscale > hscale);
		
		// New width and height.
		int nwidth = 0;
		int nheight = 0;
		
		// Determine the new image size.
		if (isWidthLarger)
		{
			nwidth = (int)Math.round(imgWidth * hscale);
			nheight = height;
		}
		else
		{
			nwidth = width;
			nheight = (int)Math.round(imgHeight * wscale);
		}
		
		return new Dimension(nwidth, nheight);
	}
}