/*

Thumbnailator - Thumbnail generation library

Copyright (c) 2008, Chris Kroells
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of its contributors may be
      used to endorse or promote products derived from this software without
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


package net.coobird.thumbnailator;

import java.awt.image.*;
import java.io.*;


/**
 * A helper class for Thumbnailator to perform quick thumbnail generation.
 * This class provides static convenience methods to provide ease of use
 * of the <code>TGenerator</code> thumbnail generator.
 * @author coobird
 *
 */
public class THelper
{
	private static TGenerator tg = new TGenerator();
	
	/**
	 * <code>THelper</code> should not be instantiated.
	 */
	private THelper()
	{
	}
	
	/**
	 * Generates a thumbnail image from a {@link BufferedImage}.
	 * @param img 					the source image
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail 
	 * @return 						the thumbnail image as an instance of
	 * 								<code>BufferedImage</code>.
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static BufferedImage createThumbnail(BufferedImage img,
												int width, int height,
												int outputOrientation,
												int outputRenderQuality)
												throws IOException
	{
		// Sets up the parameters.
		TImageParams param = new TImageParams();
		param.setInputImage(img);
		param.setOutputDimension(width, height);
		param.setOutputOrientation(outputOrientation);
		param.setOutputRenderQuality(outputRenderQuality);
		
		// Create the thumbnail.
		tg.createThumbnail(param);
		
		// Return the thumbnail as BufferedImage.
		return param.getOutputImage();
	}

	/**
	 * Generates a thumbnail image from a {@link BufferedImage}.
	 * @param img 					the source image
	 * @param out 					the destination file to output the
	 * 								thumbnail to
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail 
	 * @return 						the thumbnail image as an instance of
	 * 								<code>BufferedImage</code>.
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static void createThumbnail(BufferedImage img,
										File out,
										int width, int height,
										int outputOrientation,
										int outputRenderQuality)
										throws IOException
	{
		// Sets up the parameters.
		TImageParams param = new TImageParams();
		param.setInputImage(img);
		param.setOutputImageFile(out);
		param.setOutputDimension(width, height);
		param.setOutputOrientation(outputOrientation);
		param.setOutputRenderQuality(outputRenderQuality);
		
		// Create the thumbnail.
		tg.createThumbnail(param);
	}
	
	/**
	 * Generates a thumbnail from a source image file and outputs to a file.
	 * @param in 					the source image file
	 * @param out 					the destination file to output the
	 * 								thumbnail to
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static void createThumbnail(File in, File out,
										int width, int height)
										throws IOException
	{
		createThumbnail(in, null, out, width, height,
				TGenerator.NO_ROTATION, TGenerator.RENDER_LOW_QUALITY,
				TGenerator.WM_UNDEFINED, 1.0f);
	}
	
	/**
	 * Generates a thumbnail from a source image file and outputs to a file.
	 * @param in 					the source image file
	 * @param out 					the destination file to output the
	 * 								thumbnail to
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail 
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static void createThumbnail(File in, File out,
										int width, int height,
										int outputOrientation,
										int outputRenderQuality)
										throws IOException
	{
		createThumbnail(in, null, out, width, height,
						outputOrientation, outputRenderQuality,
						TGenerator.WM_UNDEFINED, 1.0f);
	}

	/**
	 * Generates a thumbnail with a watermark from a source image file
	 * and outputs to a file.
	 * @param in 					the source image file
	 * @param wm 					the watermark image file
	 * @param out 					the destination file to output the
	 * 								thumbnail to
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail
	 * @param watermarkPosition		the positioning of the watermark in the
	 * 								thumbnail. Constants are defined in the
	 * 								{@link TGenerator} class.
	 * @param watermarkAlpha 		the transparency level of the watermark
	 * 								A <code>float</code> value between
	 * 								0.0 to 1.0 (both inclusive) where 0.0 is
	 * 								transparent, and 1.0 is opaque.
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static void createThumbnail(File in, File wm, File out,
											int width, int height,
											int outputOrientation,
											int outputRenderQuality,
											int watermarkPosition,
											float watermarkAlpha)
											throws IOException
		{
			// Sets up the parameters.
			TImageParams param = new TImageParams();
			param.setInputImageFile(in);
			param.setInputWatermarkFile(wm);
			param.setOutputImageFile(out);
			param.setOutputDimension(width, height);
			param.setOutputOrientation(outputOrientation);
			param.setOutputRenderQuality(outputRenderQuality);
			param.setWatermarkPosition(watermarkPosition);
			param.setWatermarkAlpha(watermarkAlpha);
			
			// Create the thumbnail.
			tg.createThumbnail(param);
			
			in = null;
			out = null;
			wm = null;
			param= null;
		}

	 
	/**
	 * Generates a thumbnail from a source image and returns a
	 * <code>BufferedImage</code>.
	 * @param in 					the source image file
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail 
	 * @return 						the thumbnail image as an instance of
	 * 								<code>BufferedImage</code>.
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static BufferedImage createThumbnail(File in,
												int width, int height,
												int outputOrientation,
												int outputRenderQuality)
												throws IOException
	{
		return createThumbnail(in, null, width, height,
						outputOrientation, outputRenderQuality,
						TGenerator.WM_UNDEFINED, 1.0f);
	}

	
	
	/**
	 * Generates a thumbnail with a watermark from a source image and returns a
	 * <code>BufferedImage</code>.
	 * @param in 					the source image file
	 * @param wm 					the watermark image file
	 * @param width 				the width of the thumbnail
	 * @param height 				the height of the thumbnail
	 * @param outputOrientation 	the orientation of the thumbnail
	 * @param outputRenderQuality	the quality setting for rendering the
	 * 								thumbnail
	 * @param watermarkPosition		the positioning of the watermark in the
	 * 								thumbnail. Constants are defined in the
	 * 								{@link TGenerator} class.
	 * @param watermarkAlpha 		the transparency level of the watermark.
	 * 								A <code>float</code> value between
	 * 								0.0 to 1.0 (both inclusive) where
	 * 								0.0 is transparent, and 1.0 is opaque.
	 * @return 						the thumbnail image as an instance of
	 * 								<code>BufferedImage</code>.
	 * @throws IOException			Thrown when a problem occurs during
	 * 								the thumbnail generation.
	 */
	public static BufferedImage createThumbnail(File in, File wm,
												int width, int height,
												int outputOrientation,
												int outputRenderQuality,
												int watermarkPosition,
												float watermarkAlpha)
												throws IOException
	{
		// Sets up the parameters.
		TImageParams param = new TImageParams();
		param.setInputImageFile(in);
		param.setInputWatermarkFile(wm);
		param.setOutputDimension(width, height);
		param.setOutputOrientation(outputOrientation);
		param.setOutputRenderQuality(outputRenderQuality);
		param.setWatermarkPosition(watermarkPosition);
		param.setWatermarkAlpha(watermarkAlpha);
		
		// Create the thumbnail.
		tg.createThumbnail(param);
		
		in = null;
		wm = null;
		
		return param.getOutputImage();
	}
}