/*

Thumbnailator - Thumbnail generation library

Copyright (c) 2008, Chris Kroells
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of its contributors may be
      used to endorse or promote products derived from this software without
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


package net.coobird.thumbnailator;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

import javax.imageio.ImageIO;


/**
 * The <code>TImageParams</code> class is a container for parameters used
 * in generating a thumbnail using Thumbnailator. An instance of this class
 * is used by <code>TGenerator</code> as the basic unit of performing a
 * thumbnail generation task. 
 * @author coobird
 *
 */
public class TImageParams implements Cloneable
{
	// Fields for a thumbnail generation task.
	private BufferedImage inputImage = null;
	private File inputImageFile = null;
	private String inputFormat = null;
	
	private BufferedImage inputWatermark = null;
	private File inputWatermarkFile = null;
	
	private BufferedImage outputImage = null;
	private File outputImageFile = null;
	private String outputFormat = null;
	
	private Dimension outputDimension = null;
	
	/**
	 * Whether or not to keep the aspect ratio of the original image
	 * to be the same in the thumbnail.
	 */
	private boolean keepAspectRatio = true;
	private boolean allowSubsampling = true;
	
	private Rectangle watermarkRectangle = null;
	private int watermarkPosition = TGenerator.WM_UNDEFINED;
	private float watermarkAlpha = 1.0f;
	
	// Default value for outputQuality is NaN.
	private float outputQuality = Float.NaN;
	private int outputRenderQuality = TGenerator.RENDER_HIGHEST_QUALITY;
	private int outputOrientation = TGenerator.NO_ROTATION;
	
	/**
	 * Sets whether or not to have the output thumbnail to have the 
	 * same file modified time as the original image.
	 */
	private boolean keepDateTime = true;

	/**
	 * Instantiates a <code>TImageParams</code> object. 
	 */
	public TImageParams()
	{
		outputDimension = new Dimension();
		watermarkRectangle = new Rectangle();
	}
	
	/**
	 * Returns a copy of the <code>TImageParams</code> object.
	 * @return 						a copy of this object.
	 */
	public Object clone() throws CloneNotSupportedException
	{
		TImageParams newObj = (TImageParams)super.clone();
		
		// Note: File object is immutable, so clone() is not necessary.
		
		if (inputImage != null)
			newObj.inputImage = (BufferedImage)this.inputImage;
		if (inputImageFile != null)
			newObj.inputImageFile = this.inputImageFile;
		if (inputWatermark != null)
			newObj.inputWatermark = (BufferedImage)this.inputWatermark;
		if (inputWatermarkFile != null)
			newObj.inputWatermarkFile = this.inputWatermarkFile;
		if (outputImage != null)
			newObj.outputImage = (BufferedImage)this.outputImage;
		if (outputImageFile != null)
			newObj.outputImageFile = this.outputImageFile;
		if (outputDimension != null)
			newObj.outputDimension = (Dimension)this.outputDimension.clone();
		if (watermarkRectangle != null)
			newObj.watermarkRectangle = (Rectangle)this.watermarkRectangle.clone();
		
		return newObj;
	}
	
	/**
	 * Returns the image that is used as the source for the thumbnail.
	 * @return 						the image to make a thumbnail from.
	 */
	public BufferedImage getInputImage()
	{
		return inputImage;
	}

	/**
	 * Sets the image that is used as a source to produce the thumbnail from.
	 * @param inputImage 			the image to make a thumbnail from.
	 */
	public void setInputImage(BufferedImage inputImage)
	{
		this.inputImage = inputImage;
	}

	/**
	 * Returns a {@link File} object that represents the source image file.
	 * @return 						the <code>File</code> object that
	 * 								represents the source image file.
	 */
	public File getInputImageFile()
	{
		return inputImageFile;
	}

	/**
	 * Sets the source image to a specified input {@link File} object.
	 * @param inputImageFile 		the source image as a <code>File</code>
	 * 								object.
	 */
	public void setInputImageFile(File inputImageFile)
	{
		this.inputImageFile = inputImageFile;
		this.setInputFormat(TImageParams.determineFormat(inputImageFile));
	}

	/**
	 * Returns a string specifying the source image format.
	 * @return 						the string of the source image format.
	 */
	public String getInputFormat()
	{
		return inputFormat;
	}

	/**
	 * Sets the source image format.
	 * The source image format is automatically set when the source
	 * file is set through the {@link #setInputImageFile(File)} method.
	 * @param inputFormat 			the source image format
	 */
	public void setInputFormat(String inputFormat)
	{
		this.inputFormat = inputFormat;
	}

	/**
	 * Returns the watermark source image.
	 * @return 						the watermark source image.
	 */
	public BufferedImage getInputWatermark()
	{
		return inputWatermark;
	}

	/**
	 * Sets the watermark source image. 
	 * @param inputWatermark 		the watermark source image
	 */
	public void setInputWatermark(BufferedImage inputWatermark)
	{
		this.inputWatermark = inputWatermark;
	}

	/**
	 * Returns the watermark source file.
	 * @return 						the watermark source file.
	 */
	public File getInputWatermarkFile()
	{
		return inputWatermarkFile;
	}

	/**
	 * Sets the watermark source file.
	 * @param inputWatermarkFile	the watermark source file
	 */
	public void setInputWatermarkFile(File inputWatermarkFile)
	{
		this.inputWatermarkFile = inputWatermarkFile;
	}

	/**
	 * Returns the output image.
	 * This method should be called upon completion of the thumbnail
	 * generation process. {@link TGenerator} sets the output image of the
	 * {@link TImageParams} object when calling a method that returns a
	 * {@link BufferedImage} thumbnail.
	 * @return 						the output image.
	 */
	public BufferedImage getOutputImage()
	{
		return outputImage;
	}

	/**
	 * Sets the output image.
	 * This method is called by {@link THelper} when {@link TGenerator}
	 * returns a {@link BufferedReader} as the thumbnail image. 
	 * @param outputImage 			the image that should be set as the
	 * 								output image
	 */
	public void setOutputImage(BufferedImage outputImage)
	{
		this.outputImage = outputImage;
	}

	/**
	 * Returns the file destination of the thumbnail.
	 * @return 						the thumbnail destination file.
	 */
	public File getOutputImageFile()
	{
		return outputImageFile;
	}

	/**
	 * Sets the file destination of the thumbnail.
	 * @param outputImageFile 		the destination file for the thumbnail
	 */
	public void setOutputImageFile(File outputImageFile)
	{
		this.outputImageFile = outputImageFile;
		this.setOutputFormat(TImageParams.determineFormat(outputImageFile));
	}

	/**
	 * Returns the output image format string.
	 * @return						the output image format string.
	 */
	public String getOutputFormat()
	{
		return outputFormat;
	}

	/**
	 * Sets the output image format string.
	 * @param outputFormat			the output image format string
	 */
	public void setOutputFormat(String outputFormat)
	{
		this.outputFormat = outputFormat;
	}

	/**
	 * Returns the width of the thumbnail image.
	 * @return						the width of the thumbnail.
	 */
	public int getOutputWidth()
	{
		return outputDimension.width;
	}

	/**
	 * Sets the width of the thumbnail image.
	 * @param outputWidth			the width of the thumbnail
	 */
	public void setOutputWidth(int outputWidth)
	{
		outputDimension.width = outputWidth;
	}

	/**
	 * Returns the height of the thumbnail image.
	 * @return						the height of the thumbnail.
	 */
	public int getOutputHeight()
	{
		return outputDimension.height;
	}

	/**
	 * Sets the height of the thumbnail image.
	 * @param outputHeight			the height of the thumbnail
	 */
	public void setOutputHeight(int outputHeight)
	{
		outputDimension.height = outputHeight;
	}

	/**
	 * Returns the dimension (width and height) of the thumbnail image.
	 * @return						the dimension of the thumbnail.
	 */
	public Dimension getOutputDimension()
	{
		return outputDimension;
	}

	/**
	 * Sets the dimension (width and height) of the thumbnail image.
	 * @param outputDimension		the dimension of the thumbnail
	 */
	public void setOutputDimension(Dimension outputDimension)
	{
		this.outputDimension = outputDimension;
	}
	
	/**
	 * Sets the dimension (width and height) of the thumbnail image.
	 * @param width					the width of the thumbnail
	 * @param height				the height of the thumbnail
	 */
	public void setOutputDimension(int width, int height)
	{
		this.outputDimension = new Dimension(width, height);
	}

	/**
	 * Returns whether or not the aspect ratio is to be preserved when
	 * generating the thumbnail.
	 * @return 						<code>true</code> when the aspect ratio is
	 * 								to be preserved, <code>false</code>
	 * 								otherwise.
	 */
	public boolean isKeepAspectRatio()
	{
		return keepAspectRatio;
	}

	/**
	 * Sets whether or not the aspect ratio of the original image is to be
	 * preserved for the thumbnail.
	 * @param keepAspectRatio		whether or not to keep the aspect ratio
	 * 								when generating the thumbnail
	 */
	public void setKeepAspectRatio(boolean keepAspectRatio)
	{
		this.keepAspectRatio = keepAspectRatio;
	}

	/**
	 * Returns the position and dimension of the watermark.
	 * @return						the position and dimension of the watermark.
	 */
	public Rectangle getWatermarkRectangle()
	{
		return watermarkRectangle;
	}
	
	/**
	 * Sets the position and dimension of the watermark.
	 * @param watermarkRectangle 	the position and dimension of the
	 * 								watermark
	 */
	public void setWatermarkRectangle(Rectangle watermarkRectangle)
	{
		this.watermarkRectangle = watermarkRectangle;
	}

	/**
	 * Returns the width of the watermark.
	 * @return						the width of the watermark.
	 */
	public int getWatermarkWidth()
	{
		return watermarkRectangle.width;
	}

	/**
	 * Sets the width of the watermark.
	 * @param watermarkWidth		the width of the watermark
	 */
	public void setWatermarkWidth(int watermarkWidth)
	{
		watermarkRectangle.width = watermarkWidth;
	}

	/**
	 * Returns the height of the watermark.
	 * @return						the height of the watermark.
	 */
	public int getWatermarkHeight()
	{
		return watermarkRectangle.height;
	}

	/**
	 * Sets the height of the watermark.
	 * @param watermarkHeight		the height of the watermark
	 */
	public void setWatermarkHeight(int watermarkHeight)
	{
		watermarkRectangle.height = watermarkHeight;
	}

	/**
	 * Returns the dimension (width and height) of the watermark.
	 * @return						the dimension of the watermark.
	 */
	public Dimension getWatermarkDimension()
	{	
		return new Dimension(watermarkRectangle.width, watermarkRectangle.height);
	}

	/**
	 * Sets the dimension (width and height) of the watermark.
	 * @param watermarkDimension 	the dimension of the watermark
	 */
	public void setWatermarkDimension(Dimension watermarkDimension)
	{
		watermarkRectangle.width = watermarkDimension.width;
		watermarkRectangle.height = watermarkDimension.height;
	}

	/**
	 * Returns the position of the watermark.
	 * @return						the position of the watermark.
	 */
	public int getWatermarkPosition()
	{
		return watermarkPosition;
	}

	/**
	 * Sets the position of the watermark.
	 * @param watermarkPosition		the position of the watermark
	 */
	public void setWatermarkPosition(int watermarkPosition)
	{
		this.watermarkPosition = watermarkPosition;
	}

	/**
	 * Returns the transparency level of the watermark.
	 * @return the watermarkAlpha 	the transparency level as a 
	 * 								<code>float</code> value between 0.0f to
	 * 								1.0f, where 0.0 is transparent and 1.0
	 * 								is opaque.
	 */
	public float getWatermarkAlpha()
	{
		return watermarkAlpha;
	}

	/**
	 * Sets the transparency level of the watermark.
	 * @param watermarkAlpha 		the transparency level of the watermark.
	 * 								A <code>float</code> value between 
	 * 								0.0 to 1.0 (both inclusive) where 0.0 is
	 * 								transparent, and 1.0 is opaque.
	 * @throws IllegalArgumentException		Thrown when the alpha is not within
	 * 										the range of 0.0f to 1.0f,
	 * 										inclusive.   
	 */
	public void setWatermarkAlpha(float watermarkAlpha)
	{
		if (watermarkAlpha < 0.0f || watermarkAlpha > 1.0f)
			throw new IllegalArgumentException("The watermark alpha must been between 0.0f and 1.0f.");
		this.watermarkAlpha = watermarkAlpha;
	}
	
	/**
	 * Returns the rendering quality for the thumbnail generation.
	 * The constants used for determining the rendering quality is defined in
	 * the {@link TGenerator} class.
	 * @return 						the output rendering quality constant.
	 */
	public int getOutputRenderQuality()
	{
		return outputRenderQuality;
	}
	
	/**
	 * Sets the render quality settings.
	 * The constants used for determining the rendering quality is defined in
	 * the {@link TGenerator} class.
	 * @param outputRenderQuality	the output rendering quality constant.
	 */
	public void setOutputRenderQuality(int outputRenderQuality)
	{
		this.outputRenderQuality = outputRenderQuality;
	}

	/**
	 * Returns the compression quality for the thumbnail.
	 * @return 						the compression quality value for the
	 * 								thumbnail. A <code>float</code> value
	 * 								between	0.0f and 1.0f, where 0.0f denotes
	 * 								the lowest compression quality and 1.0f
	 * 								denotes the highest compression quality.
	 */
	public float getOutputQuality()
	{
		return outputQuality;
	}

	/**
	 * Sets the compression quality for the thumbnail.
	 * @param outputQuality 		the output compression quality. An 
	 * 								<code>float</code> value between 0.0f and
	 * 								1.0f, where 0.0f denotes the lowest
	 * 								compression	quality and 1.0f denotes the
	 * 								highest	compression quality.
	 * @throws IllegalArgumentException	Thrown when the compression quality
	 * 									value is not within the range of
	 * 									0.0f to 1.0f, inclusive.   
	 */
	public void setOutputQuality(float outputQuality)
	{
		if (outputQuality < 0.0f || outputQuality > 1.0f)
			throw new IllegalArgumentException("The output quality must be between 0.0f and 1.0f.");
		
		this.outputQuality = outputQuality;
	}

	/**
	 * Returns the orientation of the thumbnail.
	 * @return						the orientation of the thumbnail. The
	 * 								constant is one of the defined orientations
	 * 								from the {@link TGenerator} class.
	 */
	public int getOutputOrientation()
	{
		return outputOrientation;
	}

	/**
	 * Sets the orientation of the thumbnail.
	 * @param outputOrientation 	the orientation of the thumbnail as a
	 * 								constant defined in the
	 * 								{@link TGenerator} class.
	 */
	public void setOutputOrientation(int outputOrientation)
	{
		this.outputOrientation = outputOrientation;
	}

	/**
	 * Returns whether or not the timestamp of the original image file is
	 * kept for the thumbnail image file.
	 * @return 						<code>true</code> if the timestamp is kept
	 * 								the same for the thumbnail image file,
	 * 								<code>false</code> otherwise.
	 */
	public boolean isKeepDateTime()
	{
		return keepDateTime;
	}

	/**
	 * Sets whether or not to keep the timestamp of the source image as the
	 * timestamp of the thumbnail image file. 
	 * @param keepDateTime		whether or not to keep the timestamp
	 */
	public void setKeepDateTime(boolean keepDateTime)
	{
		this.keepDateTime = keepDateTime;
	}

	/**
	 * Returns whether or not subsampling is enabled when reading the
	 * source image.
	 * @return						<code>true</code> when subsampling is
	 * 								enabled, <code>false</code> if subsampling
	 * 								is disabled.
	 */
	public boolean isAllowSubsampling()
	{
		return allowSubsampling;
	}

	/**
	 * Sets whether or not subsampling is enabled when reading the
	 * source image.
	 * @param allowSubsampling		whether or not subsampling is enabled
	 */
	public void setAllowSubsampling(boolean allowSubsampling)
	{
		this.allowSubsampling = allowSubsampling;
	}
	
	/**
	 * Determines the file format from the extension of the file name.
	 * Note that this method only checks the extension of the specified
	 * file in order to determine the file format. It does not examine
	 * the file itself.
	 * @param f 					a file for which the format is to be
	 * 								determined.
	 * @return 						the file format of the file as a
	 * 								<code>String</code>. Returns
	 * 								<code>null</code> if format cannot be
	 * 								determined.
	 */
	public static String determineFormat(File f)
	{
		if (f == null)
			return null;
		
		String filename = f.getName();
		String ext = null;
		
		if (filename.lastIndexOf('.') != -1)
			ext = filename.substring(filename.lastIndexOf('.') + 1);
		else
			return null;
		
		// Checks if the current ImageReaders support the extension of the
		// file whose format is being determined.
		String[] names = ImageIO.getReaderFormatNames();
		for (int i = 0; i < names.length; i++)
			if (names[i].equals(ext))
				return ext;
		
		// Could not determine the format.
		return null;
	}
}