/*

Thumbnailator - Thumbnail generation library

Copyright (c) 2008, Chris Kroells
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the author nor the names of its contributors may be
      used to endorse or promote products derived from this software without
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


package net.coobird.thumbnailator;

import java.awt.*;
import java.io.*;
import java.util.*;


/**
 * <p>
 * The <code>TRunner</code> class is used to perform multiple thumbnail
 * generation tasks. It accepts {@link TImageParams} objects for the 
 * tasks to perform.
 * </p>
 * <p>
 * Only one instance of {@link TGenerator} is instantiated when an instance of
 * the <code>TRunner</code> is created in order to generate thumbnails.
 * </p>
 * <p>
 * The <code>TRunner</code> class operates in the following matter:
 * </p>
 * <ol>
 *		<li>
 * 		A <code>TRunner</code> object is instantiated with the default
 * 		constructor or by the {@link TRunner#TRunner(TImageParams)} constructor.
 * 		</li>
 * 		<li>
 * 		One of the <code>add</code> methods is called to add a thumbnail
 * 		generation task to the list of tasks to perform.
 * 		</li>
 * 		<li>
 * 		The last parameters (as a <code>TImageParams</code> object) is
 * 		retrieved to use as a template for the parameters to use for this
 * 		new thumbnail generation task. (If there were no parameters before
 * 		this task, as in the case of the first task, a new instance of
 * 		<code>TImageParams</code> is produced.)
 * 		</li>
 * 		<li>
 * 		The last parameters is updated with the new parameters for the
 * 		current task, and then the parameters is added to the list of tasks
 * 		to perform. Internally, the tasks are stored as a 
 * 		{@link java.util.LinkedList}.
 * 		</li>
 * 		<li>
 * 		Once all the tasks to be performed are added to the instance of
 * 		<code>TRunner</code>, the {@link #start()} method is called.
 * 		</li>
 * 		<li>
 * 		The <code>start()</code> method the instantiates a
 * 		<code>TGenerator</code>, sends the <code>TImageParams</code>
 * 		one by one to the <code>TGenerator</code> instance to perform the
 * 		thumbnail generation.
 * 		</li>
 * 		<li>
 * 		The list of tasks is cleared.
 * 		</li>
 * </ol>  
 * @author coobird
 *
 */
public class TRunner
{
	private LinkedList tasks = null;
	private TImageParams lastParam = null;
	
	/**
	 * Instantiates a <code>TRunner</code> object.
	 */
	public TRunner()
	{
		tasks = new LinkedList();
	}
	
	/**
	 * Instantiates a <code>TRunner</code> object with the current parameters
	 * set to the parameters specified in the {@link TImageParams} object. 
	 * @param param				the <code>TImageParams</code> object to set
	 * 							the current parameters to.
	 */
	public TRunner(TImageParams param)
	{
		lastParam = param;
	}
	
	/**
	 * Adds a the parameters of a thumbnail generation task (as a
	 * {@link TImageParams} object) to the list of tasks to perform.
	 * @param param 			a <code>TImageParams</code> object which
	 * 							contains the parameters to use for a thumbnail
	 * 							generation task
	 */
	public void add(TImageParams param)
	{
		try
		{
			tasks.add(param.clone());
			lastParam = (TImageParams)param.clone();
		}
		catch (CloneNotSupportedException e)
		{
			e.printStackTrace(System.err);
		}
	}
	
	/**
	 * Removes a thumbnail generation task ({@link TImageParams}) from
	 * the list of tasks to perform.
	 * @param param 			a thumbnail generation task to remove
	 * @return 					<code>true</code> if the task is removed,
	 * 							<code>false</code> if it is not removed.
	 */
	public boolean remove(TImageParams param)
	{
		return tasks.remove(param);
	}
	
	/**
	 * Adds a thumbnail generation task (as a {@link TImageParams} object) to
	 * the list of tasks to perform.
	 * @param inFile 			the source image file
	 * @param outFile			the thumbnail destination file
	 * @param d					the dimension of the thumbnail
	 */
	public void add(File inFile, File outFile, Dimension d)
	{
		add(inFile, outFile, d.width, d.height);
	}

	/**
	 * Adds the parameters of a thumbnail generation task to the list of
	 * tasks to perform.
	 * @param inFile 			the source image file
	 * @param outFile			the thumbnail destination file
	 * @param width				the width of the thumbnail
	 * @param height			the height of the thumbnail
	 */
	public void add(File inFile, File outFile, int width, int height)
	{
		TImageParams param = null;
		
		// If the last used parameters doesn't exist, then make a new instance.
		// If exists, then load that up as default paramters.
		if (lastParam == null)
			param = new TImageParams();
		else
			param = lastParam;
		
		param.setInputImageFile(inFile);
		param.setOutputImageFile(outFile);
		param.setOutputDimension(width, height);
		
		add(param);
	}

	/**
	 * Adds the parameters of a thumbnail generation task to the list of
	 * tasks to perform.
	 * @param inFile 			the source image file
	 * @param wmFile			the watermark source image file
	 * @param outFile			the thumbnail destination file
	 * @param width				the width of the thumbnail
	 * @param height			the height of the thumbnail
	 * @param orientation 		the orientation of the thumbnail image with
	 * 							respect to the original image.
	 * 							The rotation mode used is set by specifying
	 * 							one of the constants that is provided by the
	 * 							{@link TGenerator} class.
	 * @param renderQuality 	the rendering quality for the thumbnail.
	 * 							The rendering quality is set by specifying
	 * 							one of the constants that is provided by the
	 * 							<code>TGenerator</code> class.
	 * @param wmPosition 		the position of the watermark on the thumbnail.
	 * 							The constants used for the positioning of the
	 * 							watermark is provided by the
	 * 							<code>TGenerator</code> class.
	 * @param wmAlpha 			the opacity level of the watermark.
	 * 							A <code>float</code> value between 0.0 to 1.0
	 * 							(both inclusive) where 0.0 is transparent,
	 * 							and 1.0 is opaque.
	 * @param compressionQuality	the quality of the compression to use for
	 * 								the thumbnail.
	 * 								A value of 0.0f indicates lowest quality
	 * 								and 1.0f indicates the highest quality
	 * 								output.
	 */
	public void add(File inFile, File wmFile, File outFile, 
					int width, int height, int orientation, int renderQuality,
					int wmPosition, float wmAlpha, float compressionQuality)
	{
		TImageParams param = null;
		
		// If the last used parameters doesn't exist, then make a new instance.
		// If exists, then load that up as default paramters.
		if (lastParam == null)
			param = new TImageParams();
		else
			param = lastParam;
		
		param.setInputImageFile(inFile);
		param.setOutputImageFile(outFile);
		param.setInputWatermarkFile(wmFile);
		param.setOutputDimension(width, height);
		param.setOutputOrientation(orientation);
		param.setOutputRenderQuality(renderQuality);
		param.setWatermarkPosition(wmPosition);
		param.setWatermarkAlpha(wmAlpha);
		param.setOutputQuality(compressionQuality);
		
		add(param);
	}

	/**
	 * Starts processing of the thumbnail generation tasks.
	 * After the tasks are performed, the list of tasks are cleared.
	 * If the list of tasks are needed for future use, one should call the
	 * {@link #getArray()} or {@link #iterator()} method before calling
	 * this method.
	 * @throws IOException		thrown when there is a problem during the
	 * 							thumbnail generation.
	 */
	public void start() throws IOException
	{
		Iterator iter = tasks.iterator();
		TGenerator tg = new TGenerator();
		
		for ( ; iter.hasNext(); )
			tg.createThumbnail((TImageParams)iter.next());
		
		tasks.clear();
	}
	
	/**
	 * Returns an iterator of thumbnail generation task parameters
	 * ({@link TImageParams}).
	 * @return 					an iterator of <code>TImageParams</code>.
	 */
	public Iterator iterator()
	{
		return tasks.iterator();
	}
	
	/**
	 * Returns an array of thumbnail generation task parameters,
	 * ({@link TImageParams}[]).
	 * @return					an array of <code>TImageParams</code> objects.
	 */
	public TImageParams[] getArray()
	{
		Object[] objArray = tasks.toArray();
		
		TImageParams[] tasksArray = new TImageParams[objArray.length];
		
		for (int i = 0; i < objArray.length; i++)
			tasksArray[i] = (TImageParams)objArray[i];
		
		return tasksArray;
	}
	
	/**
	 * Returns the currrent parameters to be used for the next thumbnail
	 * generation task as a {@link TImageParams} object.
	 * @return					the current parameters for thumbnail generation
	 * 							tasks.
	 */
	public TImageParams getParams()
	{
		return lastParam;
	}
	
	/**
	 * Sets the parameters to use for a thumbnail generation task, passed in
	 * as a {@link TImageParams}.
	 * @param param 			the parameters to use for the thumbnail
	 * 							generation tasks.
	 */
	public void setParams(TImageParams param)
	{
		lastParam = param;
	}
}